@@ -1,5 +1,6 @@
 import json
 import os
+import random
 from typing import Any, Dict
 
 import httpx
@@ -13,6 +14,8 @@ from pydantic_ai.providers.google_gla import GoogleGLAProvider
 from pydantic_ai.providers.openai import OpenAIProvider
 from pydantic_ai.providers.openrouter import OpenRouterProvider
 
+from code_puppy.tools.common import console
+
 # Environment variables used in this module:
 # - GEMINI_API_KEY: API key for Google's Gemini models. Required when using Gemini models.
 # - OPENAI_API_KEY: API key for OpenAI models. Required when using OpenAI models or custom_openai endpoints.
@@ -23,6 +26,59 @@ from pydantic_ai.providers.openrouter import OpenRouterProvider
 #   Example: "X-Api-Key": "$OPENAI_API_KEY" will use the value from os.environ.get("OPENAI_API_KEY")
 
 
+def build_proxy_dict(proxy):
+    proxy_tokens = proxy.split(":")
+    structure = "{}:{}@{}:{}".format(
+        proxy_tokens[2], proxy_tokens[3], proxy_tokens[0], proxy_tokens[1]
+    )
+    proxies = {
+        "http": "http://{}/".format(structure),
+        "https": "http://{}".format(structure),
+    }
+    return proxies
+
+
+def build_httpx_proxy(proxy):
+    """Build an httpx.Proxy object from a proxy string in format ip:port:username:password"""
+    proxy_tokens = proxy.split(":")
+    if len(proxy_tokens) != 4:
+        raise ValueError(
+            f"Invalid proxy format: {proxy}. Expected format: ip:port:username:password"
+        )
+
+    ip, port, username, password = proxy_tokens
+    proxy_url = f"http://{ip}:{port}"
+    proxy_auth = (username, password)
+
+    # Log the proxy being used
+    console.log(f"Using proxy: {proxy_url} with username: {username}")
+
+    return httpx.Proxy(url=proxy_url, auth=proxy_auth)
+
+
+def get_random_proxy_from_file(file_path):
+    """Reads proxy file and returns a random proxy formatted for httpx.AsyncClient"""
+    if not os.path.exists(file_path):
+        raise ValueError(f"Proxy file '{file_path}' not found.")
+
+    with open(file_path, "r") as f:
+        proxies = [line.strip() for line in f.readlines() if line.strip()]
+
+    if not proxies:
+        raise ValueError(
+            f"Proxy file '{file_path}' is empty or contains only whitespace."
+        )
+
+    selected_proxy = random.choice(proxies)
+    try:
+        return build_httpx_proxy(selected_proxy)
+    except ValueError:
+        console.log(
+            f"Warning: Malformed proxy '{selected_proxy}' found in file '{file_path}', ignoring and continuing without proxy."
+        )
+        return None
+
+
 def get_custom_config(model_config):
     custom_config = model_config.get("custom_endpoint", {})
     if not custom_config:
@@ -97,7 +153,18 @@ class ModelFactory:
 
         elif model_type == "custom_anthropic":
             url, headers, ca_certs_path, api_key = get_custom_config(model_config)
-            client = httpx.AsyncClient(headers=headers, verify=ca_certs_path)
+
+            # Check for proxy configuration
+            proxy_file_path = os.environ.get("CODE_PUPPY_PROXIES")
+            proxy = None
+            if proxy_file_path:
+                proxy = get_random_proxy_from_file(proxy_file_path)
+
+            # Only pass proxy to client if it's valid
+            client_args = {"headers": headers, "verify": ca_certs_path}
+            if proxy is not None:
+                client_args["proxy"] = proxy
+            client = httpx.AsyncClient(**client_args)
             anthropic_client = AsyncAnthropic(
                 base_url=url,
                 http_client=client,
@@ -162,7 +229,18 @@ class ModelFactory:
 
         elif model_type == "custom_openai":
             url, headers, ca_certs_path, api_key = get_custom_config(model_config)
-            client = httpx.AsyncClient(headers=headers, verify=ca_certs_path)
+
+            # Check for proxy configuration
+            proxy_file_path = os.environ.get("CODE_PUPPY_PROXIES")
+            proxy = None
+            if proxy_file_path:
+                proxy = get_random_proxy_from_file(proxy_file_path)
+
+            # Only pass proxy to client if it's valid
+            client_args = {"headers": headers, "verify": ca_certs_path}
+            if proxy is not None:
+                client_args["proxy"] = proxy
+            client = httpx.AsyncClient(**client_args)
             provider_args = dict(
                 base_url=url,
                 http_client=client,
